RW = LibStub("AceAddon-3.0"):NewAddon("Raid Watch", "AceEvent-3.0", "AceConsole-3.0")
if not RW then return end

--[===[@debug@
RW.debug = true
--@end-debug@]===]

-- Lua funcs
local pairs, tonumber, error, type = pairs, tonumber, error, type
local tostring = tostring

-- Version
RW.VERSION = 2.0
RW.REVISION = tonumber(("$Rev 1 $"):sub(6, -3))

-- Bits used to control usage of plugins (Credit to Bigwigs)
RW.BITS = {
	BAR 	= 0x00000001,
	MESSAGE = 0x00000002,
	WARNING = 0x00000004,
	ICON 	= 0x00000008,
	RANGE 	= 0x00000010,
}

-- Spellname lookup
RW.SpellName = setmetatable({}, {
	__index = function(t, key)
		if type(key) ~= "number" then return key end
		local name = GetSpellInfo(key)
		if not name then error(("Cannot fetch spell name for %q"):format(key)) end
		return name		
	end}
)

function RW:OnInitialize()
	local defaults = {profile = {
		vicotorySound = "RW: Victory"
	}}
	self.db = LibStub("AceDB-3.0"):New("RaidWatch2DB", defaults, "Default")
	self:SetUpLDB()
	
	self.modStorage = {}
	self.bossStorage = {}
	
	self:RegisterMessage("RW_BossModFound")
	self:RegisterMessage("RW_BossRegistered")
end

function RW:OnEnable()
	self:RegisterMessage("RW_BossModFound")
	self:RegisterMessage("RW_BossRegistered")
	
	self:RegisterChatCommand("mrw", "SlashHandler")
end

function RW:OnDisable()
end

-- General message function, usualy embedded in plugins 
-- and bosses to automaticly supply their own prefix.
function RW:Msg(msg, prefix)
	local prefix = prefix or self:GetName()
	DEFAULT_CHAT_FRAME:AddMessage(("|cff28579d[|r|cffcfe846%s|r|cff28579d]|r "):format(tostring(prefix))..msg, 0.39, 0.67, 0.87)
end

---------------------------------------------
----- Modules handling 
---------------------------------------------

-- Plugins
RW.plugins = RW:NewModule("Plugins")
RW.plugins:SetDefaultModuleState(true)
RW.plugins:SetDefaultModuleLibraries("AceEvent-3.0", "AceTimer-3.0")
--- Register a plugin
-- @param name The name of the module to register
-- @returns The created module
function RW:Plugin(name, ...)
	local module = RW.plugins:NewModule(name, ...)
	module.Msg = self.Msg
	return module
end

-- Bosses
RW.bosses = RW:NewModule("Bosses")
RW.bosses:SetDefaultModuleState(false)
RW.bosses:SetDefaultModuleLibraries("AceEvent-3.0", "AceTimer-3.0")
--- Register a boss module
-- @param name Name of the boss
-- @param loadZone Zone/Zones where this boss module should be enabled/tracking
-- @param modPack The mod package this boss belongs to
-- @param subGroup If passed it will be used in the options to sub group (Wings in Naxx for example)
-- @param ... Default libraries to embed for the boss
-- @return The boss module
function RW:Boss(name, loadZone, modPack, subGroup, ...)
	local boss = RW.bosses:NewModule(name, ...)
	boss.modPack = modPack
	boss.loadZone = (type(loadZone) == "table") and loadZone or {loadZone}
	boss.Msg = self.Msg
	boss.subGroup = subGroup
	-- TODO: Do this in some better way
	-- This is to handle if the boss module is loaded bu LoD or if it is run in one folder
	-- i.e. via svn install
	if self:GetPlugin("LoD").loadZones and self:GetPlugin("LoD").loadZones[loadZone] then
		boss.LoD = true
	end
	return boss
end

function RW:GetPlugin(name)	return RW.plugins:GetModule(name, true) end
function RW:IteratePlugins() return RW.plugins:IterateModules() end
function RW:GetBoss(name) return RW.bosses:GetModule(name, true) end
function RW:IterateBosses() return RW.bosses:IterateModules() end

function RW:RegisterBoss(boss)
	local LoD = self:GetPlugin("LoD")
	if boss.LoD then
		for zone, zoneData in pairs(LoD.loadZones) do
			for i, mod in pairs(zoneData) do
				if mod.name == boss.modPack then
					boss.id = mod.modId
				end
			end
		end
	else
		self:SendMessage("RW_BossModFound", boss.modPack)
		self:SendMessage("RW_NewMonitorZone", boss.loadZone)
	end
	
	if boss.OnRegister then boss:OnRegister() end
	boss.db = self.db:RegisterNamespace(boss:GetName(), {profile = boss.defaultOptions })	
	self:SendMessage("RW_BossRegistered", boss)
end

function RW:RegisterPlugin(plugin)
	if plugin.OnRegister then plugin:OnRegister() end
	self:SendMessage("RW_PluginRegistered", plugin)
end

function RW:RW_BossModFound(message, name, id)
	self.modStorage[name] = id
end

function RW:RW_BossRegistered(message, boss)
	self.bossStorage[boss] = true
end

function RW:GetModsAndBosses()
	for name, id in pairs(self.modStorage) do
		self:SendMessage("RW_BossModFound", name, id)
	end
	for boss in pairs(self.bossStorage) do
		self:SendMessage("RW_BossRegistered", boss)
	end
end

--------------------------------------------
-- Slash commands

function RW:SlashHandler(input)
	if input == "" or input == "config" then
		local config = self:GetPlugin("Options")
		if not config then
			if IsAddOnLoaded("RW_Options") then
				config = self:GetPlugin("Options")
			else 	
				LoadAddOn("RW_Options")
				if IsAddOnLoaded("RW_Options") then
					config = self:GetPlugin("Options")
				else 
					self:Msg("Could not open options, |cFF10FF32RW-GUI|r could not be found")
				end
			end
		end
		if config then config:Toggle() end
	end
end
---------------------------------------------
-- LDB

function RW:SetUpLDB()
	local LDB = LibStub("LibDataBroker-1.1")
	self.object = LDB:NewDataObject("Raid Watch", {
		type = "launcher",
		icon = "Interface\\Icons\\INV_Axe_94",
		OnClick = function(clickedframe, button)
			if button == "LeftButton" then
				self:SlashHandler("config")
			end
		end,
		OnTooltipShow = function(tooltip)
			if not tooltip or not tooltip.AddLine then return end
			tooltip:AddLine("|cff30c345Raid Watch|r")
			tooltip:AddLine("Click to toggle config mode")
		end,
		label = "Raid Watch",
		})
	
end
